/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015, 2016  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.*;

import org.gfxtest.framework.*;
import org.gfxtest.framework.annotations.*;

/**
 * This test renders bitmap filled by given color.
 *
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.RENDER_TEST)
@GraphicsPrimitive(GraphicsPrimitives.COMMON_BITMAP)
@RenderStyle(RenderStyles.NONE)
@Transformation(Transformations.NONE)
@Zoom(1)
public class BlankImage extends GfxTest
{

    /**
     * Basic test of graphics test framework - it just passes and does not erase
     * nor fill the image.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBasicTest(TestImage image, Graphics2D graphics)
    {
        return TestResult.PASSED;
    }

    /**
     * This test fill image with pure black color.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithBlackColor(TestImage image, Graphics2D graphics)
    {
        image.fillImage(Color.BLACK);
        return TestResult.PASSED;
    }

    /**
     * This test fill image with pure white color.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithWhiteColor(TestImage image, Graphics2D graphics)
    {
        image.fillImage(Color.WHITE);
        return TestResult.PASSED;
    }

    /**
     * This test fill image with pure red color.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithRedColor(TestImage image, Graphics2D graphics)
    {
        image.fillImage(Color.RED);
        return TestResult.PASSED;
    }

    /**
     * This test fill image with pure green color.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithGreenColor(TestImage image, Graphics2D graphics)
    {
        image.fillImage(Color.GREEN);
        return TestResult.PASSED;
    }

    /**
     * This test fill image with pure blue color.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithBlueColor(TestImage image, Graphics2D graphics)
    {
        image.fillImage(Color.BLUE);
        return TestResult.PASSED;
    }

    /**
     * This test fill image with mixed color.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithMixedColor1(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(0x7f, 0x7f, 0x00));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with mixed color.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithMixedColor2(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(0.5f, 0.0f, 1.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent red color with 0% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentRedColor1(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 0.0f, 0.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent red color with 33% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentRedColor2(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 0.0f, 0.33f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent red color with 66% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentRedColor3(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 0.0f, 0.66f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent red color with 100% opacity.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentRedColor4(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 0.0f, 1.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent blue color with 0% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentBlueColor1(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(0.0f, 0.0f, 1.0f, 0.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent blue color with 33% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentBlueColor2(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(0.0f, 0.0f, 1.0f, 0.33f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent blue color with 66% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentBlueColor3(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(0.0f, 0.0f, 1.0f, 0.66f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent blue color with 100% opacity.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentBlueColor4(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(0.0f, 0.0f, 1.0f, 1.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent green color with 0% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentGreenColor1(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 0.0f, 0.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent green color with 33% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentGreenColor2(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 0.0f, 0.33f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent green color with 66% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentGreenColor3(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 0.0f, 0.66f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent green color with 100% opacity.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentGreenColor4(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 0.0f, 1.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent magenta color with 0% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentMagentaColor1(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 1.0f, 0.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent magenta color with 33% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentMagentaColor2(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 1.0f, 0.33f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent magenta color with 66% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentMagentaColor3(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 1.0f, 0.66f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent magenta color with 100% opacity.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentMagentaColor4(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 0.0f, 1.0f, 1.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent black color with 0% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentBlackColor1(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(0.0f, 0.0f, 0.0f, 0.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent black color with 33% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentBlackColor2(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(0.0f, 0.0f, 0.0f, 0.33f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent black color with 66% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentBlackColor3(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(0.0f, 0.0f, 0.0f, 0.66f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent black color with 100% opacity.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentBlackColor4(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(0.0f, 0.0f, 0.0f, 1.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent white color with 0% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentWhiteColor1(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 1.0f, 1.0f, 0.0f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent white color with 33% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentWhiteColor2(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 1.0f, 1.0f, 0.33f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent white color with 66% opacity.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentWhiteColor3(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 1.0f, 1.0f, 0.66f));
        return TestResult.PASSED;
    }

    /**
     * This test fill image with semitransparent white color with 100% opacity.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testFillWithSemiTransparentWhiteColor4(TestImage image, Graphics2D graphics)
    {
        image.fillImage(new Color(1.0f, 1.0f, 1.0f, 1.0f));
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     *
     * @param args not used in this case
     */
    public static void main(String[] args)
    {
        new BlankImage().runTestSuite(args);
    }
}
