/*
  Java gfx-test framework

   Copyright (C) 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.Graphics2D;



import org.gfxtest.callbacks.CommonArcDrawCallbacks;
import org.gfxtest.callbacks.CommonCircleDrawCallbacks;
import org.gfxtest.callbacks.DiagonalLineDrawCallbacks;
import org.gfxtest.callbacks.HorizontalLineDrawCallbacks;
import org.gfxtest.callbacks.VerticalLineDrawCallbacks;
import org.gfxtest.framework.ColorPalette;
import org.gfxtest.framework.PrintTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.RenderStyle;
import org.gfxtest.framework.annotations.RenderStyles;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



/**
 * This test renders filled shapes using ColorPaint.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.RENDER_TEST)
@RenderStyle(RenderStyles.FILL)
@Transformation(Transformations.NONE)
@Zoom(1)
public class PrintTestColorPaint extends PrintTest
{

    /**
     * Method which renders set of horizontal lines using various colors and
     * stroke styles. For each line, the callback function/method is called to
     * perform all required setup.
     * 
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param verticalStep
     *            between two near lines
     * @param horizontalLineDrawCallbacks
     *            class containing set of callback methods
     */
    private static void drawHorizontalLines(TestImage image, Graphics2D graphics, int verticalStep, HorizontalLineDrawCallbacks horizontalLineDrawCallbacks)
    {
        // setup rendering
        horizontalLineDrawCallbacks.setup(image, graphics);

        // image width and height
        final int width = image.getWidth();
        final int height = image.getHeight();

        // horizontal coordinates of line endpoints
        final int x1 = BORDER;
        final int x2 = width - BORDER;

        // index to color palette
        int colorIndex = 0;

        // draw all lines onto a paper
        for (int y = 0; y < height; y += verticalStep)
        {
            // setup can be made for each line
            horizontalLineDrawCallbacks.iterationCallBack(y, colorIndex++);
            // render the line
            graphics.drawLine(x1, y, x2, y);
        }

        // cleanup rendering
        horizontalLineDrawCallbacks.cleanup();
    }

    /**
     * Method which renders set of vertical lines using various colors and
     * stroke styles. For each line, the callback function/method is called to
     * perform all required setup.
     * 
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param horizontalStep
     *            between two near lines
     * @param verticalLineDrawCallbacks
     *            class containing set of callback methods
     */
    private static void drawVerticalLines(TestImage image, Graphics2D graphics, int horizontalStep, VerticalLineDrawCallbacks verticalLineDrawCallbacks)
    {
        // setup rendering
        verticalLineDrawCallbacks.setup(image, graphics);

        // image width and height
        final int width = image.getWidth();
        final int height = image.getHeight();

        // horizontal coordinates of line endpoints
        final int y1 = BORDER;
        final int y2 = height - BORDER;

        // index to color palette
        int colorIndex = 0;

        // draw all lines onto a paper
        for (int x = 0; x < width; x += horizontalStep)
        {
            // setup can be made for each line
            verticalLineDrawCallbacks.iterationCallBack(x, colorIndex++);
            // render the line
            graphics.drawLine(x, y1, x, y2);
        }

        // cleanup rendering
        verticalLineDrawCallbacks.cleanup();
    }

    /**
     * Method which renders set of diagonal lines using various colors and
     * stroke styles. For each line, the callback function/method is called to
     * perform all required setup.
     * 
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param diagonalStep
     *            between two near lines
     * @param diagonalLineDrawCallbacks
     *            class containing set of callback methods
     */
    private static void drawDiagonalLines(TestImage image, Graphics2D graphics, int diagonalStep, DiagonalLineDrawCallbacks diagonalLineDrawCallbacks)
    {
        // setup rendering
        diagonalLineDrawCallbacks.setup(image, graphics);

        // image width and height
        final int width = image.getWidth();
        final int height = image.getHeight();

        // length of the "path"
        final int length = width + height - BORDER * 8;

        // coordinates of the first line
        int x1 = BORDER;
        int y1 = BORDER;
        int x2 = BORDER;
        int y2 = BORDER;

        // index to color palette
        int colorIndex = 0;

        // draw all lines onto a paper
        for (int d = 0; d < length; d += DIAGONAL_STEP)
        {
            // first endpoint
            if (x1 < width - BORDER * 2)
            {
                x1 += DIAGONAL_STEP;
            }
            else
            {
                y1 += DIAGONAL_STEP;
            }
            // second endpoint
            if (y2 < height - BORDER * 2)
            {
                y2 += DIAGONAL_STEP;
            }
            else
            {
                x2 += DIAGONAL_STEP;
            }
            // setup can be made for each line
            diagonalLineDrawCallbacks.iterationCallBack(d, colorIndex++);
            // render the line
            graphics.drawLine(x1, y1, x2, y2);
        }

        // cleanup rendering
        diagonalLineDrawCallbacks.cleanup();
    }

    /**
     * Method which renders set of circles using various colors and
     * stroke styles. For each circle, the callback function/method is called to
     * perform all required setup.
     * 
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param radiusStep
     *            between two near circles
     * @param circleDrawCallback
     *            class containing set of callback methods
     */
    private void drawCircle(TestImage image, Graphics2D graphics2d, int radiusStep, CommonCircleDrawCallbacks circleDrawCallback)
    {
        // setup rendering
        circleDrawCallback.setup(image, graphics2d);

        // image width and height
        final int width = image.getWidth();
        final int height = image.getHeight();

        // horizontal coordinates of circle center
        final int xc = width >> 1;
        final int yc = height >> 1;

        // maximum radius
        final int maxRadius = Math.min(width, height) - BORDER;

        // index to color palette
        int colorIndex = 0;

        // draw all circles onto a paper
        for (int radius = MINIMUM_RADIUS; radius < maxRadius; radius += radiusStep)
        {
            // setup can be made for each circle
            circleDrawCallback.iterationCallBack(xc, yc, radius, maxRadius, colorIndex++);
            // render the circle
            graphics2d.drawOval(xc - radius, yc - radius, radius << 1, radius << 1);
        }

        // cleanup rendering
        circleDrawCallback.cleanup();
    }

    /**
     * Method which renders set of arcs using various colors and
     * stroke styles. For each arc, the callback function/method is called to
     * perform all required setup.
     * 
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param radiusStep
     *            between two near arcs
     * @param arcDrawCallback
     *            class containing set of callback methods
     */
    private void drawArcs(TestImage image, Graphics2D graphics2d, int radiusStep, CommonArcDrawCallbacks arcDrawCallback)
    {
        // setup rendering
        arcDrawCallback.setup(image, graphics2d);

        // image width and height
        final int width = image.getWidth();
        final int height = image.getHeight();

        // horizontal coordinates of arc center
        final int xc = width >> 1;
        final int yc = height >> 1;

        // maximum radius
        final int maxRadius = Math.min(width, height) - BORDER;

        // index to color palette
        int colorIndex = 0;

        // arc start angle
        int startAngle = 0;

        // draw all arcs onto a paper
        for (int radius = MINIMUM_RADIUS; radius < maxRadius; radius += radiusStep)
        {
            // setup can be made for each arc
            arcDrawCallback.iterationCallBack(xc, yc, radius, maxRadius, colorIndex++);
            startAngle += 4;
            // render the arc
            graphics2d.drawArc(xc - radius, yc - radius, radius << 1, radius << 1, startAngle, 270);
        }

        // cleanup rendering
        arcDrawCallback.cleanup();
    }

    /**
     * Test basic behavior of method Graphics.drawLine().
     * Horizontal lines are rendered with default width and default end caps.
     * Lines color are selected from a palette.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawHorizontalLinesColorPalette(TestImage image, Graphics2D graphics)
    {
        drawHorizontalLines(image, graphics, VERTICAL_STEP, new HorizontalLineDrawCallbacks()
        {
            /**
             * Callback function called before each line is rendered.
             */
            @Override
            public void iterationCallBack(int y, int index)
            {
                // set line color
                this.graphics.setColor(ColorPalette.getColor(index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawLine().
     * Vertical lines are rendered with default width and default end caps.
     * Lines color are selected from a palette.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawVerticalLinesColorPalette(TestImage image, Graphics2D graphics)
    {
        drawVerticalLines(image, graphics, HORIZONTAL_STEP, new VerticalLineDrawCallbacks()
        {
            /**
             * Callback function called before each line is rendered.
             */
            @Override
            public void iterationCallBack(int x, int index)
            {
                // set line color
                this.graphics.setColor(ColorPalette.getColor(index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }
    
    /**
     * Test basic behavior of method Graphics.drawLine().
     * Diagonal lines are rendered with default width and default end caps.
     * Lines color are selected from a palette.
     *
     * @param image
     *            image to which lines are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawDiagonalLinesColorPalette(TestImage image, Graphics2D graphics)
    {
        drawDiagonalLines(image, graphics, DIAGONAL_STEP, new DiagonalLineDrawCallbacks()
        {
            /**
             * Callback function called before each line is rendered.
             */
            @Override
            public void iterationCallBack(int x, int index)
            {
                // set line color
                this.graphics.setColor(ColorPalette.getColor(index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles color are selected from a palette.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleColorPalette(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP, new CommonCircleDrawCallbacks()
        {
            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set new circle color to index
                this.graphics.setColor(ColorPalette.getColor(index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawOval().
     * Circles color are selected from a palette.
     *
     * @param image
     *            image to which circles are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawCircleColorPaletteInv(TestImage image, Graphics2D graphics2d)
    {
        drawCircle(image, graphics2d, CIRCLE_RADIUS_STEP, new CommonCircleDrawCallbacks()
        {
            /**
             * Callback function called before each circle is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set new circle color according to index
                this.graphics.setColor(ColorPalette.getColor(MAX_COLOR_INDEX - index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arc color are selected from a palette.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcColorPalette(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP, new CommonArcDrawCallbacks()
        {
            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set arc color
                this.graphics.setColor(ColorPalette.getColor(index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawArc().
     * Arc color are selected from a palette.
     *
     * @param image
     *            image to which arcs are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawArcColorPaletteInv(TestImage image, Graphics2D graphics2d)
    {
        drawArcs(image, graphics2d, ARC_RADIUS_STEP, new CommonArcDrawCallbacks()
        {
            /**
             * Callback function called before each arc is rendered.
             */
            @Override
            public void iterationCallBack(int x, int y, int radius, int maxRadius, int index)
            {
                // set arc color
                this.graphics.setColor(ColorPalette.getColor(MAX_COLOR_INDEX - index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     * 
     * @param args
     *            not used in this case
     */
    public static void main(String[] args)
    {
        new PrintTestColorPaint().runTestSuite(args);
    }
}
